<?php

declare(strict_types=1);

namespace Drupal\simple_interactive_maps\Form;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\CloseModalDialogCommand;
use Drupal\Core\Ajax\MessageCommand;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Url;
use Drupal\simple_interactive_maps\Entity\InteractiveMap;
use Drupal\simple_interactive_maps\GroupTableGeneratorTrait;
use Drupal\simple_interactive_maps\InteractiveMapFormTempStoreTrait;
use Drupal\simple_interactive_maps\MapDataLoader;

/**
 * Provides a confirmation form for deleting a group from a map.
 */
final class GroupDeleteConfirmForm extends ConfirmFormBase {

  use InteractiveMapFormTempStoreTrait, GroupTableGeneratorTrait;

  /**
   * The map that the group will be deleted from.
   *
   * @var \Drupal\simple_interactive_maps\Entity\InteractiveMap|null
   */
  protected ?InteractiveMap $map;

  /**
   * The group to be deleted.
   *
   * @var string|null
   */
  protected ?string $group;

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'simple_interactive_maps_group_delete_confirm';
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion(): TranslatableMarkup {
    return $this->t('Are you sure you want to do delete this group?');
  }

  /**
   * {@inheritdoc}
   */
  public function getDescription() {
    return $this->t('This will permanently delete the group from this map and cannot be undone.');
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl(): Url {
    return new Url('system.admin_config');
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, InteractiveMap $interactive_map = NULL, string $group = NULL): array {
    $this->map = $interactive_map;
    $this->group = $group;

    $form = parent::buildForm($form, $form_state);

    $form['actions']['submit']['#ajax'] = [
      'callback' => '::submitForm',
    ];

    $form['actions']['cancel'] = [
      '#type' => 'button',
      '#value' => $this->t('Cancel'),
      '#ajax' => [
        'callback' => '::cancelAjax',
      ],
    ];

    $form['map'] = [
      '#type' => 'value',
      '#value' => $interactive_map,
    ];

    $form['group'] = [
      '#type' => 'value',
      '#value' => $group,
    ];

    return $form;
  }

  /**
   * Cancel the form and close the modal dialog.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   The AJAX response.
   */
  public function cancelAjax(array &$form, FormStateInterface $form_state): AjaxResponse {
    $response = new AjaxResponse();

    // Close the modal dialog.
    $response->addCommand(new CloseModalDialogCommand());

    return $response;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): AjaxResponse {
    $response = new AjaxResponse();

    $tempStore = $this->getMapTempStore($this->map);
    $groups = $tempStore->get(MapDataLoader::MAP_TEMP_STORE_GROUPS_KEY);
    unset($groups[$this->group]);
    $tempStore->set(MapDataLoader::MAP_TEMP_STORE_GROUPS_KEY, $groups);

    // Close the modal dialog.
    $response->addCommand(new CloseModalDialogCommand());

    // Generate the groups table.
    $mapGroupForm = new GroupManagementForm();
    $updatedGroupsTable = $mapGroupForm->generateGroupsTable($groups, $this->map);

    // Replace the old groups table with the updated one.
    $response->addCommand(new ReplaceCommand('#groups-table', $updatedGroupsTable));
    $response->addCommand(new MessageCommand('You have unsaved changes.', 'warning', ['sticky' => TRUE], TRUE));
    $response->addCommand(new MessageCommand('The group has been deleted from the map.'));

    return $response;
  }

}
