<?php

declare(strict_types=1);

namespace Drupal\simple_interactive_maps\Form;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\ChangedCommand;
use Drupal\Core\Ajax\CloseModalDialogCommand;
use Drupal\Core\Ajax\MessageCommand;
use Drupal\Core\Ajax\RemoveCommand;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\TempStore\TempStoreException;
use Drupal\simple_interactive_maps\GroupTableGeneratorTrait;
use Drupal\simple_interactive_maps\InteractiveMapFormTempStoreTrait;
use Drupal\simple_interactive_maps\InteractiveMapInterface;
use Drupal\simple_interactive_maps\MapDataLoader;
use Drupal\simple_interactive_maps\SimpleInteractiveMapsLoggerTrait;
use Psr\Container\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\HttpException;

/**
 * Provides a Simple Interactive Maps form.
 */
final class GroupModalForm extends FormBase {

  use GroupTableGeneratorTrait, InteractiveMapFormTempStoreTrait, SimpleInteractiveMapsLoggerTrait;

  /**
   * Form Constructor.
   *
   * @param \Drupal\Core\Form\FormBuilderInterface $formBuilder
   *   Form builder service.
   */
  public function __construct(protected FormBuilderInterface $formBuilder) {}

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container): GroupModalForm {
    $formBuilder = $container->get('form_builder');

    return new GroupModalForm(
      $formBuilder
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'simple_interactive_maps_group_modal';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, InteractiveMapInterface $interactive_map = NULL, string $group = NULL): array {
    if (!($interactive_map instanceof InteractiveMapInterface)) {
      throw new HttpException(404, 'Interactive map not found.');
    }

    $form['map'] = [
      '#type' => 'value',
      '#value' => $interactive_map,
    ];

    if ($group !== NULL) {
      $tempStore = $this->getMapTempStore($interactive_map);
      $groups = $tempStore->get(MapDataLoader::MAP_TEMP_STORE_GROUPS_KEY);
      if (!isset($groups[$group])) {
        throw new HttpException(404, 'Group not found.');
      }
      $form['group'] = [
        '#type' => 'value',
        '#value' => $group,
      ];
      $group = $groups[$group];
    }

    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Name'),
      '#required' => TRUE,
      '#default_value' => $group ? $group['label'] : '',
    ];

    // Create a machine name derived from the name field.
    $form['machine_name'] = [
      '#type' => 'machine_name',
      '#title' => $this->t('Machine name'),
      '#required' => TRUE,
      '#default_value' => $group ? $group['machine_name'] : '',
      '#machine_name' => [
        'source' => ['label'],
        'exists' => [$this, 'groupExists'],
      ],
      '#element_validate' => [
        [$this, 'validateMachineNameFormat'],
      ],
    ];

    $form['tabs'] = [
      '#type' => 'horizontal_tabs',
      '#default_tab' => 'edit-group',
    ];

    $form['tabs']['colors'] = [
      '#type' => 'details',
      '#title' => $this->t('Colors'),
      '#group' => 'tabs',
      '#attributes' => ['class' => ['edit-group']],
    ];

    $form['tabs']['colors']['override_colors'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Override colors'),
      '#default_value' => $group ? $group['override_colors'] : FALSE,
    ];

    $form['tabs']['colors']['fill_color'] = [
      '#type' => 'color',
      '#title' => $this->t('Fill color'),
      '#default_value' => $group ? $group['fill_color'] : $interactive_map->get('fill_color'),
      '#states' => [
        'visible' => [
          ':input[name="override_colors"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['tabs']['colors']['hover_color'] = [
      '#type' => 'color',
      '#title' => $this->t('Hover color'),
      '#default_value' => $group ? $group['hover_color'] : $interactive_map->get('hover_color'),
      '#states' => [
        'visible' => [
          ':input[name="override_colors"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['tabs']['colors']['stroke_color'] = [
      '#type' => 'color',
      '#title' => $this->t('Stroke color'),
      '#default_value' => $group ? $group['stroke_color'] : $interactive_map->get('stroke_color'),
      '#states' => [
        'visible' => [
          ':input[name="override_colors"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['tabs']['tooltip'] = [
      '#type' => 'details',
      '#title' => $this->t('Tooltip'),
      '#group' => 'tabs',
      '#attributes' => ['class' => ['edit-group']],
    ];

    $form['tabs']['tooltip']['override_tooltip'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Override tooltip'),
      '#default_value' => $group ? $group['override_tooltip'] : FALSE,
    ];

    $form['tabs']['tooltip']['tooltip'] = [
      '#type' => 'text_format',
      '#title' => $this->t('Tooltip'),
      '#default_value' => $group ? $group['tooltip']['value'] : '',
      '#format' => $group ? $group['tooltip']['format'] : 'full_html',
      '#states' => [
        'visible' => [
          ':input[name="override_tooltip"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['tabs']['actions'] = [
      '#type' => 'details',
      '#title' => $this->t('Actions'),
      '#group' => 'tabs',
      '#attributes' => ['class' => ['edit-group']],
    ];

    $form['tabs']['actions']['override_action'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Override action'),
      '#default_value' => $group ? $group['override_action'] : FALSE,
    ];

    $form['tabs']['actions']['action'] = [
      '#type' => 'radios',
      '#title' => $this->t('Action'),
      '#options' => [
        'none' => $this->t('None'),
        'url' => $this->t('URL'),
        'content' => $this->t('Content'),
      ],
      '#default_value' => $group['action'] ?: 'none',
      '#states' => [
        'visible' => [
          ':input[name="override_action"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['tabs']['actions']['url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('URL'),
      '#default_value' => $group ? $group['url'] : '',
      '#states' => [
        'visible' => [
          ':input[name="action"]' => ['value' => 'url'],
          ':input[name="override_action"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['tabs']['actions']['content'] = [
      '#type' => 'text_format',
      '#title' => $this->t('Content'),
      '#default_value' => $group ? $group['content']['value'] : '',
      '#format' => $group ? $group['content']['format'] : 'full_html',
      '#states' => [
        'visible' => [
          ':input[name="action"]' => ['value' => 'content'],
          ':input[name="override_action"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['actions'] = [
      '#type' => 'actions',
      'submit' => [
        '#type' => 'submit',
        '#value' => $this->t('Save'),
        '#ajax' => [
          'callback' => '::submitForm',
          'event' => 'click',
          'progress' => [
            'type' => 'throbber',
            'message' => $this->t('Saving Group Info...'),
          ],
          'prevent' => 'submit',
        ],
      ],
      'cancel' => [
        '#type' => 'button',
        '#value' => $this->t('Cancel'),
        '#limit_validation_errors' => [],
        '#ajax' => [
          'callback' => '::closeModalForm',
          'event' => 'click',
          'progress' => [
            'type' => 'throbber',
            'message' => $this->t('Closing Form...'),
          ],
          'prevent' => 'submit',
        ],
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {}

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): AjaxResponse {
    $response = new AjaxResponse();
    $map = $form_state->getValue('map');
    if ($map instanceof InteractiveMapInterface) {
      $tempStore = $this->getMapTempStore($map);
      $groups = $tempStore->get(MapDataLoader::MAP_TEMP_STORE_GROUPS_KEY);

      $group = [
        'label' => $form_state->getValue('label'),
        'override_colors' => $form_state->getValue('override_colors'),
        'fill_color' => $form_state->getValue('fill_color'),
        'hover_color' => $form_state->getValue('hover_color'),
        'stroke_color' => $form_state->getValue('stroke_color'),
        'override_tooltip' => $form_state->getValue('override_tooltip'),
        'tooltip' => $form_state->getValue('tooltip'),
        'override_action' => $form_state->getValue('override_action'),
        'url' => $form_state->getValue('url'),
        'content' => $form_state->getValue('content'),
        'action' => $form_state->getValue('action'),
      ];

      if ($form_state->getValue('group')) {
        $groups[$form_state->getValue('machine_name')] = $group;
        $rowSelector = '#row-group-' . strtolower($form_state->getValue('group'));
        $asteriskSelector = sprintf('.form-item--groups-%s-label', strtolower($form_state->getValue('group')));
        $response->addCommand(new ChangedCommand($rowSelector, $asteriskSelector));
      }
      else {
        $groups[$form_state->getValue('machine_name')] = $group;
      }

      try {
        $tempStore->set(MapDataLoader::MAP_TEMP_STORE_GROUPS_KEY, $groups);
      }
      catch (TempStoreException $e) {
        $this->getMapsLogger()->error($e->getMessage());
      }

      /* @phpstan-ignore-next-line */
      $groupsForm = $this->formBuilder->getForm(GroupManagementForm::class, $map);
      $table = $groupsForm['groups'];
      $response->addCommand(new ReplaceCommand('#group-table', $table), TRUE);
    }
    $response->addCommand(new RemoveCommand('.messages-list__item'));
    $response->addCommand(new MessageCommand('You have unsaved changes.', NULL, [
      'type' => 'warning',
      'sticky' => FALSE,
    ], TRUE));
    $response->addCommand(new MessageCommand('Group details for ' . $form_state->getValue('label') . ' saved.', NULL, [
      'type' => 'status',
      'sticky' => FALSE,
    ], FALSE));
    $response->addCommand(new CloseModalDialogCommand());
    return $response;
  }

  /**
   * Closes the modal form.
   *
   * @param array $form
   *   The form structure.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   The AJAX response.
   */
  public function closeModalForm(array &$form, FormStateInterface $form_state): AjaxResponse {
    $response = new AjaxResponse();
    $response->addCommand(new CloseModalDialogCommand());
    return $response;
  }

  /**
   * Validates the machine name format.
   *
   * @param array $element
   *   The form element.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   * @param array $form
   *   The complete form structure.
   */
  public function validateMachineNameFormat(array &$element, FormStateInterface $form_state, array $form) {
    $machine_name = $form_state->getValue('machine_name');
    $machine_name = strtolower($machine_name);
    $machine_name = str_replace(' ', '_', $machine_name);
    $machine_name = preg_replace('/[^a-z0-9_]/', '', $machine_name);
    $form_state->setValue('machine_name', $machine_name);
  }

  /**
   * Checks if a group with the given machine name already exists.
   *
   * @param string $machine_name
   *   The machine name to check.
   * @param array $element
   *   The form element.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   *
   * @return bool
   *   TRUE if the group exists, FALSE otherwise.
   */
  public function groupExists(string $machine_name, array $element, FormStateInterface $form_state): bool {
    $tempStore = $this->getMapTempStore($form_state->getValue('map'));
    $groups = $tempStore->get(MapDataLoader::MAP_TEMP_STORE_GROUPS_KEY);

    return array_key_exists($machine_name, $groups);
  }

}
