<?php

declare(strict_types=1);

namespace Drupal\simple_interactive_maps\Form;

use Drupal\Core\Config\Entity\ConfigEntityStorageInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\TempStore\TempStoreException;
use Drupal\Core\Url;
use Drupal\simple_interactive_maps\InteractiveMapFormTempStoreTrait;
use Drupal\simple_interactive_maps\InteractiveMapInterface;
use Drupal\simple_interactive_maps\MapDataLoader;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\HttpException;

/**
 * Provides a Simple Interactive Maps form.
 */
final class MapRegionsForm extends FormBase {

  use InteractiveMapFormTempStoreTrait;

  /**
   * Constructs a new MapRegionsForm object.
   *
   * @param \Drupal\Core\Config\Entity\ConfigEntityStorageInterface $mapStorage
   *   The map storage.
   * @param \Drupal\Core\Form\FormBuilderInterface $formBuilder
   *   The form builder.
   */
  public function __construct(protected ConfigEntityStorageInterface $mapStorage, protected FormBuilderInterface $formBuilder) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): MapRegionsForm {

    /** @var \Drupal\Core\Config\Entity\ConfigEntityStorageInterface $mapStorage */
    $mapStorage = $container->get('entity_type.manager')
      ->getStorage('interactive_map');

    /** @var \Drupal\Core\Form\FormBuilderInterface $formBuilder */
    $formBuilder = $container->get('form_builder');

    return new MapRegionsForm($mapStorage, $formBuilder);
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'simple_interactive_maps_map_regions';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, InteractiveMapInterface $interactive_map = NULL): array {
    if ($interactive_map === NULL) {
      throw new HttpException(404, 'Map not found.');
    }

    $regionDetails = $interactive_map->get('regions') ?: [];

    $regionTempStorage = $this->getMapTempStore($interactive_map);
    if ($regionTempStorage->getMetadata(MapDataLoader::MAP_TEMP_STORE_REGIONS_KEY) === NULL) {
      try {
        $regionTempStorage->set(MapDataLoader::MAP_TEMP_STORE_REGIONS_KEY, $regionDetails);
      }
      catch (TempStoreException) {
        throw new HttpException(500, 'Failed to store region details.');
      }
    }

    $form['id'] = [
      '#type' => 'hidden',
      '#value' => $interactive_map->id(),
    ];

    $form['actions'] = [
      '#type' => 'actions',
      'submit' => [
        '#type' => 'submit',
        '#value' => $this->t('Save'),
      ],
    ];

    $form['regions'] = [
      '#type' => 'table',
      '#header' => [
        $this->t('Region'),
        $this->t('Group'),
        $this->t('Edit'),
      ],
    ];

    foreach ($regionDetails as $id => $region) {
      $form['regions'][$id] = [
        '#attributes' => ['class' => ['region-form-row'], 'id' => ['row-region-' . strtolower($id)]],
      ];

      $form['regions'][$id]['label'] = [
        '#type' => 'item',
        '#markup' => $region['label'],
      ];

      $groups = $interactive_map->get('groups') ?: [];

      $form['regions'][$id]['group'] = [
        '#type' => 'item',
        '#markup' => $region['group'] ? $groups[$region['group']]['label'] : '',
      ];

      $form['regions'][$id]['edit-link'] = [
        '#type' => 'link',
        '#title' => $this->t('Edit'),
        '#url' => Url::fromRoute('simple_interactive_maps.map_region_modal_form', [
          'interactive_map' => $interactive_map->id(),
          'regionId' => $id,
        ]),
        '#attributes' => [
          'class' => ['use-ajax'],
          'data-dialog-type' => 'modal',
          'data-dialog-options' => \json_encode([
            'width' => 800,
            'minHeight' => 650,
            'title' => $this->t('Editing @region Details', ['@region' => $region['label']]),
          ], JSON_THROW_ON_ERROR),
        ],
      ];
    }

    $form['#attached']['library'][] = 'core/drupal.dialog.ajax';

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {}

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $map = $this->mapStorage->load($form_state->getValue('id'));

    if ($map instanceof InteractiveMapInterface) {
      $regionTempStorage = $this->getMapTempStore($map);
      $regionDetails = $regionTempStorage->get(MapDataloader::MAP_TEMP_STORE_REGIONS_KEY);
      $map->set('regions', $regionDetails);
      $map->save();
      $this->messenger()->addStatus($this->t('The map has been updated.'));
      $regionTempStorage->delete(MapDataLoader::MAP_TEMP_STORE_REGIONS_KEY);
    }
    else {
      $this->messenger()->addError($this->t('Failed to update the map.'));
    }

    $route_name = $this->getRouteMatch()->getRouteName();
    $route_parameters = $this->getRouteMatch()->getRawParameters()->all();
    $url = Url::fromRoute($route_name, $route_parameters);
    $form_state->setRedirectUrl($url);
  }

}
