<?php

declare(strict_types=1);

namespace Drupal\simple_interactive_maps\Form;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\ChangedCommand;
use Drupal\Core\Ajax\CloseModalDialogCommand;
use Drupal\Core\Ajax\MessageCommand;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\TempStore\TempStoreException;
use Drupal\simple_interactive_maps\InteractiveMapFormTempStoreTrait;
use Drupal\simple_interactive_maps\InteractiveMapInterface;
use Drupal\simple_interactive_maps\MapDataLoader;
use Drupal\simple_interactive_maps\SimpleInteractiveMapsLoggerTrait;
use Symfony\Component\HttpKernel\Exception\HttpException;

/**
 * Provides a Simple Interactive Maps form.
 */
final class RegionModalForm extends FormBase {

  use InteractiveMapFormTempStoreTrait, SimpleInteractiveMapsLoggerTrait;

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'simple_interactive_maps_region_modal';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, InteractiveMapInterface $interactive_map = NULL, string $regionId = NULL): array {
    if (!($interactive_map instanceof InteractiveMapInterface)) {
      throw new HttpException(500, 'Interactive map not found.');
    }

    $regionTempStorage = $this->getMapTempStore($interactive_map);
    $regionData = $regionTempStorage->get(MapDataLoader::MAP_TEMP_STORE_REGIONS_KEY);
    $region = $regionData[$regionId];

    $form['map'] = [
      '#type' => 'value',
      '#value' => $interactive_map,
    ];

    $form['id'] = [
      '#type' => 'hidden',
      '#value' => $regionId,
    ];

    $form['label'] = [
      '#type' => 'value',
      '#value' => $region['label'],
    ];

    $form['region'] = [
      '#type' => 'fieldset',
      '#tree' => TRUE,
    ];

    $form['region']['tabs'] = [
      '#type' => 'horizontal_tabs',
      '#default_tab' => 'edit-general',
    ];

    $form['region']['tabs']['general'] = [
      '#type' => 'details',
      '#title' => $this->t('General'),
      '#group' => 'tabs',
      '#attributes' => ['class' => ['edit-general']],
    ];

    $form['region']['tabs']['general']['tooltip'] = [
      '#type' => 'text_format',
      '#title' => $this->t('Tooltip text'),
      '#default_value' => $region['tooltip']['value'] ?? '',
      '#format' => $region['tooltip']['format'] ?? 'full_html',
    ];

    $form['region']['tabs']['general']['hidden'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Hidden'),
      '#default_value' => $region['hidden'] ?: FALSE,
    ];

    $groups = $interactive_map->get('groups');

    $groupOptions = [];
    $groupActive = FALSE;
    if (!empty($groups)) {
      foreach ($groups as $groupId => $group) {
        $groupOptions[$groupId] = $group['label'];
      }

      $groupActive = TRUE;
    }

    $form['region']['tabs']['general']['group'] = [
      '#type' => 'select',
      '#title' => $this->t('Group'),
      '#options' => $groupOptions,
      '#empty_option' => '- None -',
      '#empty_value' => '',
      '#default_value' => $region['group'] ?? '',
      '#disabled' => !$groupActive,
    ];

    $form['region']['tabs']['color'] = [
      '#type' => 'details',
      '#title' => $this->t('Colors'),
      '#group' => 'tabs',
      '#attributes' => ['class' => ['edit-color']],
    ];

    $form['region']['tabs']['color']['fill_color'] = [
      '#type' => 'color',
      '#title' => $this->t('Fill Color'),
      '#default_value' => $region['fill_color'] ?? '#4ba0a6',
      '#description' => $this->t('The color to fill the region with.'),
    ];

    $form['region']['tabs']['color']['hover_color'] = [
      '#type' => 'color',
      '#title' => $this->t('Hover color'),
      '#default_value' => $region['hover_color'] ?? '#076369',
      '#description' => $this->t('The color to fill the region with when hovered over.'),
    ];

    $form['region']['tabs']['color']['stroke_color'] = [
      '#type' => 'color',
      '#title' => $this->t('Stroke color'),
      '#default_value' => $region['stroke_color'] ?? '#076369',
      '#description' => $this->t('The color to fill the region with when hovered over.'),
    ];

    $form['region']['tabs']['behavior'] = [
      '#type' => 'details',
      '#title' => $this->t('Behaviors'),
      '#group' => 'tabs',
      '#attributes' => ['class' => ['edit-behaviors']],
    ];

    $form['region']['tabs']['behavior']['action'] = [
      '#type' => 'radios',
      '#title' => $this->t('Action'),
      '#options' => [
        'none' => $this->t('None'),
        'url' => $this->t('Navigate to URL'),
        'content' => $this->t('Display Content'),
      ],
      '#default_value' => $region['action'] ?? 'none',
    ];

    $form['region']['tabs']['behavior']['url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Target URL'),
      '#default_value' => $region['url'] ?? '',
      '#states' => [
        'visible' => [
          ':input[name="region[tabs][behavior][action]"]' => ['value' => 'url'],
        ],
      ],
    ];

    $form['region']['tabs']['behavior']['content'] = [
      '#type' => 'text_format',
      '#title' => $this->t('Content'),
      '#default_value' => $region['content']['value'] ?? '',
      '#format' => $region['content']['format'] ?? 'full_html',
      '#states' => [
        'visible' => [
          ':input[name="region[tabs][behavior][action]"]' => ['value' => 'content'],
        ],
      ],
    ];

    $form['actions'] = [
      '#type' => 'actions',
      'submit' => [
        '#type' => 'submit',
        '#value' => $this->t('Save Region Info'),
        '#button_type' => 'primary',
        '#ajax' => [
          'callback' => '::submitForm',
          'event' => 'click',
          'progress' => [
            'type' => 'throbber',
            'message' => $this->t('Saving Region Info...'),
          ],
          'prevent' => 'submit',
        ],
      ],
      'cancel' => [
        '#type' => 'button',
        '#value' => $this->t('Cancel'),
        '#limit_validation_errors' => [],
        '#ajax' => [
          'callback' => '::closeModalForm',
          'event' => 'click',
          'progress' => [
            'type' => 'throbber',
            'message' => $this->t('Closing Form...'),
          ],
          'prevent' => 'submit',
        ],
      ],
    ];

    $form['#attached']['library'][] = 'core/drupal.dialog.ajax';

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {}

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): AjaxResponse {
    $regionTempStorage = $this->getMapTempStore($form_state->getValue('map'));

    $regionDetails = $regionTempStorage->get(MapDataLoader::MAP_TEMP_STORE_REGIONS_KEY);
    $regionId = $form_state->getValue('id');
    $newRegionDetails = [
      'id' => $form_state->getValue('id'),
      'label' => $form_state->getValue('label'),
      'hidden' => $form_state->getValue('region')['tabs']['general']['hidden'],
      'group' => $form_state->getValue('region')['tabs']['general']['group'],
      'fill_color' => $form_state->getValue('region')['tabs']['color']['fill_color'],
      'hover_color' => $form_state->getValue('region')['tabs']['color']['hover_color'],
      'stroke_color' => $form_state->getValue('region')['tabs']['color']['stroke_color'],
      'tooltip' => $form_state->getValue('region')['tabs']['general']['tooltip'],
      'action' => $form_state->getValue('region')['tabs']['behavior']['action'],
      'url' => $form_state->getValue('region')['tabs']['behavior']['url'],
      'content' => $form_state->getValue('region')['tabs']['behavior']['content'],
    ];
    $regionDetails[$regionId] = $newRegionDetails;

    try {
      $regionTempStorage->set(MapDataLoader::MAP_TEMP_STORE_REGIONS_KEY, $regionDetails);
    }
    catch (TempStoreException $e) {
      $this->getMapsLogger()->error($e->getMessage());
    }

    $response = new AjaxResponse();
    $response->addCommand(new CloseModalDialogCommand(FALSE));

    $rowSelector = sprintf('#row-region-%s', strtolower($regionId));
    $selector = sprintf('#edit-regions-%s-label', strtolower($regionId));
    $response->addCommand(new ChangedCommand($rowSelector, $selector));
    $response->addCommand(new MessageCommand('Region details for ' . $newRegionDetails['label'] . ' saved.', NULL, ['type' => 'status'], TRUE));
    return $response;
  }

  /**
   * Close the modal form.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   The ajax response.
   */
  public function closeModalForm(array &$form, FormStateInterface $form_state): AjaxResponse {
    $response = new AjaxResponse();
    $response->addCommand(new CloseModalDialogCommand());
    return $response;
  }

}
