<?php

declare(strict_types=1);

namespace Drupal\simple_interactive_maps;

/**
 * Load map data from the source data file.
 */
final class MapDataLoader {

  /**
   * The key for the groups data in the temp store.
   */
  public const MAP_TEMP_STORE_GROUPS_KEY = 'groups';

  /**
   * The key for the region data in the temp store.
   */
  public const MAP_TEMP_STORE_REGIONS_KEY = 'regions';

  public function __construct(private MapDefinitionPluginManager $mapDefinitionPluginManager) {}

  /**
   * Load the map data into an array.
   *
   * @return array
   *   The map data.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function loadMapData(InteractiveMapInterface $map = NULL): array {

    /** @var \Drupal\simple_interactive_maps\MapDefinitionInterface $plugin */
    $plugin = $this->mapDefinitionPluginManager->createInstance($map->get('base_map'));

    $data = $plugin->mapData();

    $this->mergeConfigData($data, $map);

    return $data;
  }

  /**
   * Merge the config data into the map data.
   *
   * @param array $mapData
   *   The map data.
   * @param \Drupal\simple_interactive_maps\InteractiveMapInterface $map
   *   The map to merge the data from.
   */
  private function mergeConfigData(array &$mapData, InteractiveMapInterface $map): void {
    $regions = $map->getRegions();
    $groups = $map->getGroups();

    array_walk($mapData, function (&$item, $key) use ($regions, $groups) {
      if (array_key_exists($key, $regions)) {
        $item = array_merge($item, $regions[$key]);
        if (($item['group'] !== '') && $groups[$item['group']]) {
          $this->applyGroupOverrides($item, $groups[$item['group']]);
        }
        $item['tooltip']['processed'] = is_array($item['tooltip']) ? check_markup($item['tooltip']['value'], $item['tooltip']['format']) : '';
        $item['content']['processed'] = is_array($item['content']) ? check_markup($item['content']['value'], $item['content']['format']) : '';
      }
    });
  }

  /**
   * Apply group overrides to the item.
   *
   * @param array $item
   *   The item to apply the overrides to.
   * @param array $group
   *   The group to apply the overrides from.
   */
  private function applyGroupOverrides(array &$item, array $group): void {
    if ($group['override_colors']) {
      $item['fill_color'] = $group['fill_color'];
      $item['hover_color'] = $group['hover_color'];
      $item['stroke_color'] = $group['stroke_color'];
    }

    if ($group['override_tooltip']) {
      $item['tooltip'] = $group['tooltip'] ?? ['value' => '', 'format' => 'plain_text'];
    }

    if ($group['override_action']) {
      $item['action'] = $group['action'];
      $item['url'] = $group['url'];
      $item['content'] = $group['content'];
    }
  }

}
